# Add-on for TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 2013 TWiki:Main/TerjeAndersen
# Copyright (C) 2008 SvenDowideit@fosiki.com
# Copyright (C) 2007-2011 TWiki Contributors. All Rights Reserved.
# TWiki Contributors are listed in the AUTHORS file in the root of
# this distribution. NOTE: Please extend that file, not this notice.
#
# This script Copyright (c) 2005 Cygnus Communications
# and distributed under the GPL (see below)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, published at 
# http://www.gnu.org/copyleft/gpl.html
#
# DeleteRevisionsAddOn.pm 
# Delete revisions by moving the .txt,v file to the trashweb, prepended with '_DELREFS_' to avoid conflicts
#

package TWiki::Contrib::DeleteRevisionsAddOn;

use strict;
use TWiki::Func;
use TWiki::Users;
use TWiki::Prefs;
use File::Basename qw(basename);

# $VERSION is referred to by TWiki, and is the only global variable that
# *must* exist in this package. It should always be Rev enclosed in dollar
# signs so that TWiki can determine the checked-in status of the plugin.
# It is used by the build automation tools, so you should leave it alone.
our $VERSION = '$Rev: 25093 (2013-02-16) $';

# This is a free-form string you can use to "name" your own plugin version.
# It is *not* used by the build automation tools, but is reported as part
# of the version number in PLUGINDESCRIPTIONS. Add a release date in ISO
# format (preferred) or a release number such as '1.3'.
our $RELEASE = '2013-02-16';

sub delrefs {
    my $session = shift;

    my $web      = $session->{webName};
    my $topic    = $session->{topicName};
    my $user     = $session->{user};

    if (&canDeleteRevisions($session, $topic, $web, $user)) {
        my $trashWeb = $TWiki::cfg{TrashWebName};

        my $dataDir = $TWiki::cfg{DataDir};
        my %relevantTrashFiles = map {basename($_) => 1} glob "$dataDir/$trashWeb/_DELREF_$web$topic*";

        my $trashTopic = $topic;

        while (defined($relevantTrashFiles{"_DELREF_$web$trashTopic.txt,v"})) {
            unless ($trashTopic =~ m/\A${topic}_?(?<trashNumber>\d+)\z/) {
                $trashTopic = "${topic}_1";
            } else {
                $trashTopic = "${topic}_" . ($+{trashNumber} + 1);
            }
            TWiki::Func::writeDebug($trashTopic);
        }

        # Move the .txt,v file to the Trash web
        my $curLoc = "$dataDir/$web/$topic.txt,v";
        my $newLoc = "$dataDir/$trashWeb/_DELREF_$web$trashTopic.txt,v";

        TWiki::Func::writeLog('delrefs',"1/2 Moving $curLoc to $newLoc",$web,$topic,$user);
        if (rename $curLoc, $newLoc) {
            TWiki::Func::writeLog('delrefs',"2/2 Saving the topic with no new content to generate new version control file.",$web,$topic,$user);

            my ($topicMeta, $topicText) = TWiki::Func::readTopic($web,$topic);
            TWiki::Func::saveTopic($web,$topic,$topicMeta,$topicText);
        } else {
            TWiki::Func::writeLog('delrefs',"1/2 Move failed, aborting...",$web,$topic,$user);
        }
    }
        
    # We now redirect the user back to the topic, revisionless or not.
    # If the referee has provided us with a redirectTo= parameter, we redirect there.
    #  - If not, we redirect to to the referee.
    #  - If the referee is not visible, we redirect to the "view" script of the topic.

    my %getParams = &getParams();
    my $url;
    if (defined($getParams{'redirectTo'})) {
        $url = TWiki::Func::getScriptUrl($web,$topic,$getParams{'redirectTo'},1);
    } elsif (defined($ENV{'HTTP_REFERER'})) {
        $url = $ENV{'HTTP_REFERER'};
    } else {
        $url = TWiki::Func::getScriptUrl($web,$topic,'view', 1);
    }
    TWiki::Func::redirectCgiQuery(undef, $url, 0); 
}

sub canDeleteRevisions {
    my ($session, $topic, $web, $user) = @_;

    return 0 unless TWiki::Func::checkAccessPermission("CHANGE", $user, undef, $topic, $web, undef);

    my $prefs = new TWiki::Prefs($session);
    my $validUsersPreference = $prefs->getTopicPreferencesValue('DELREF_VALID_USERS', $web, $TWiki::cfg{WebPrefsTopicName}) // '';
    my $validTopicsPreference = $prefs->getTopicPreferencesValue('DELREF_VALID_TOPICS', $web, $TWiki::cfg{WebPrefsTopicName}) // '';
    my $validWebsPreference = $prefs->getTopicPreferencesValue('DELREF_VALID_WEBS', $TWiki::cfg{UsersWebName}, $TWiki::cfg{SitePrefsTopicName}) // '';

    my %validTopics = map {&trim($_) => 1 } split(/,/, $validTopicsPreference);
    my %validWebs   = map {&trim($_) => 1 } split(/,/, $validWebsPreference);

    my $users = new TWiki::Users($session); 

    # We only want to proceed if the user is defined in the DELREF_VALID_USERS setting and the topic in the DELREF_VALID_TOPICS setting, as well as the web in the DELREF_VALID_WEBS in a SystemWeb (typically TWiki) preference.
    # For the users we use the TWiki::Users isInList sub, which expands groups in the list and check the user against it, as well as stripping off a 'Web.' prefix to the username (Main.terjeandersen => terjeandersen)
    # Note: This settings should be finalized in Web.WebPreferences (users and topics) and in TWiki.TWikiPreferences (web)

    return $users->isInList($user, $validUsersPreference) && exists $validTopics{$topic} && exists $validWebs{$web};
}

sub getParams {
    my %getParams;
    if (length ($ENV{'QUERY_STRING'}) > 0){
        my $buffer = $ENV{'QUERY_STRING'};
        my @pairs = split(/&/, $buffer);
        foreach my $pair (@pairs){
            my ($name, $value) = split(/=/, $pair);
            $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
            $getParams{$name} = $value;
        }
    }
    return %getParams;
}

sub trim {
    my ($x) = @_;
    s/^\s+//, s/\s+$// for $x;
    return $x;
}

1;
